package weka.associations;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;

import weka.associations.Utility.DBTidlists;
import weka.core.FastVector;
import weka.core.Instances;

public class TilingNoOverlapDatabase {
  
  private Map<Integer, Set<Integer>> tidlists;
  private Map<Integer, Integer> sizes;
  private Set<Integer> cond;
  
  public TilingNoOverlapDatabase(Instances instances) {
    DBTidlists db = Utility.makeTidlists(instances);
    this.tidlists = db.tidlists;
    this.sizes = db.sizes;
    this.cond = new TreeSet<Integer>();
  }
  
  public FastVector[] CallTiling (int k) {
		Vector<Integer> sizes = new Vector<Integer> ();
    Vector<Set<Integer>> result = this.mine(k, sizes);
    FastVector[] copy = new FastVector[result.size()+1];

		FastVector copySizes = new FastVector(sizes.size());
		for (int i = 0; i < sizes.size(); i++) {
			copySizes.addElement(sizes.get(i));
		}

		copy[0] = copySizes;

    for (int i = 0; i < result.size(); i++) {
      copy[i+1] = Utility.SetToFastVector(result.get(i));
    }
    return copy;
  }
  
  public Vector<Set<Integer>> mine(int k, Vector<Integer> resultSizes) {
    Vector<Set<Integer>> tiling = new Vector<Set<Integer>>();
    for (int i = 0; i < k; i++) {
      LTMDatabase db = new LTMDatabase(this.tidlists, this.sizes, this.cond);
      Vector<Integer> sizes = new Vector<Integer> (1);
      Set<Integer> largest = db.topK(1, sizes).firstElement();
      if (largest.size() > 0) {
      	tiling.add(largest);
				resultSizes.add(sizes.firstElement());
      } else {
      	break;
      }
      
      Iterator<Integer> it = largest.iterator();
      Set<Integer> intersection = this.tidlists.get(it.next());
      while (it.hasNext()) {
      	Set<Integer> tidlist = this.tidlists.get(it.next());
      	intersection = Utility.intersectSets(intersection, tidlist);
      }
      
      it = largest.iterator();
      Set<Integer> copy = new TreeSet<Integer> (intersection);
      while (it.hasNext()) {
      	Set<Integer> tidlist = this.tidlists.get(it.next());
      	Utility.diffSets(tidlist, copy);
      }
      
      it = intersection.iterator();
      while (it.hasNext()) {
      	int key = it.next();
      	Integer size = this.sizes.get(key);
      	this.sizes.remove(key);
      	this.sizes.put(key, size - largest.size());
      }
    }
    
    return tiling;
  }
}
