package weka.associations;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.Vector;

import weka.core.FastVector;
import weka.core.Instances;

public class Utility {
	
  /**
   * Creates a file with a given filename and writes a given text in it
   * @param filename
   * @param content
   * @throws IOExceptrion
   */
  static public void toFile(String filename, String content) throws IOException {
    FileWriter writer = new FileWriter(filename);
    BufferedWriter buffWriter = new BufferedWriter(writer);
    buffWriter.write(content);
    buffWriter.close();
    writer.close();
  }
  
  /**
   * Deletes a file
   * @param filename the name of the file
   * @throws IOException
   */
  static public void deleteFile(String filename) throws IOException {
    File file = new File(filename);
    file.delete();
  }
  
  /**
   * Build a set from a given array
   * @param array
   * @return
   */
  static public Set<Integer> buildSet(Integer[] array) {
    Set<Integer> set = new TreeSet<Integer> ();
    for (int i = 0; i < array.length; i++) {
      set.add(array[i]);
    }
    return set;
  }
  
  /**
   * Intersects 2 sets
   * @param tidlist1
   * @param tidlist2
   * @return the intersection
   */
  static public Set<Integer> intersectSets(Set<Integer> set1, Set<Integer> set2) {
    Set<Integer> intersection = new TreeSet<Integer> ();
    
    Iterator<Integer> it1 = set1.iterator();
    Iterator<Integer> it2 = set2.iterator();
    
    int transaction1 = -1;
    int transaction2 = -1;
    while (it1.hasNext()) {
      transaction1 = it1.next(); // for every item of the first set...
      
      // check whether the second set has the same item
      while ((it2.hasNext()) && (transaction1 > transaction2)) {
      	transaction2 = it2.next();
      }
      if (transaction1 == transaction2) {
      	intersection.add(transaction1);
      }
    }
    
    return intersection;
  }
  
  static public void diffSets(Set<Integer> set1, Set<Integer> set2) {
    Iterator<Integer> it1 = set1.iterator();
    Iterator<Integer> it2 = set2.iterator();
    
    int transaction1 = -1;
    int transaction2 = -1;
    while (it1.hasNext()) {
      transaction1 = it1.next(); // for every item of the first set...
      
      // check whether the second set has the same item
      while ((it2.hasNext()) && (transaction1 > transaction2)) {
	transaction2 = it2.next();
      }
      if (transaction1 == transaction2) {
	it1.remove();
      }
    }
  }
  
  /**
   * Adds a list to the first list
   * @param list1
   * @param list2
   */
  static public void addList(Vector<Set<Integer>> list1, Vector<Set<Integer>> list2) {
    for (int i = 0; i < list2.size(); i++) {
      list1.add(list2.get(i));
    }
  }
  
  static public DBTidlists makeTidlists(Instances instances) {
    // create a tidlist for every item
    Map<Integer, Set<Integer>> tidlists = new TreeMap<Integer, Set<Integer>> ();
    for (int itemNr = 0; itemNr < instances.numAttributes(); itemNr++) {
      Set<Integer> tidlist = new TreeSet<Integer>();
      tidlists.put(itemNr, tidlist);
    }
    
    // initialize the array for the transaction-sizes
    Map<Integer, Integer> sizes = new TreeMap<Integer, Integer> ();
    
    // fill up the right tidlist when a 1 is encountered, and increase the transaction's size
    for (int transaction = 0; transaction < instances.numInstances(); transaction++) {
      int transaction_size = 0;
      for (int itemNr = 0; itemNr < instances.numAttributes(); itemNr++) {
      	if (instances.instance(transaction).value(itemNr) > 0) {
      	  tidlists.get(itemNr).add(transaction);
      	  transaction_size++;
      	}
      }
      sizes.put(transaction, transaction_size);
    }
    
    DBTidlists db = new DBTidlists(sizes, tidlists);
    
    return db;
  }
  
  public static class DBTidlists {
    public Map<Integer, Integer> sizes;
    public Map<Integer, Set<Integer>> tidlists;
    
    public DBTidlists(Map<Integer, Integer> sizes, Map<Integer, Set<Integer>> tidlists) {
      this.sizes = sizes;
      this.tidlists = tidlists;
    }
  }
  
  static public FastVector SetToFastVector(Set<Integer> set) {
    FastVector copy = new FastVector(set.size());
    Iterator<Integer> it = set.iterator();
    while (it.hasNext()) {
      copy.addElement(it.next());
    }
    return copy;
  }
}
